import { Plugin } from 'ckeditor5/src/core';
import { toWidget } from 'ckeditor5/src/widget';
import { type ElementCreatorFunction as DowncastElementCreatorFunction } from '@ckeditor/ckeditor5-engine/src/conversion/downcasthelpers';
import { type ElementCreatorFunction as UpcastElementCreatorFunction } from '@ckeditor/ckeditor5-engine/src/conversion/upcasthelpers';
import { type NodeAttributes } from '@ckeditor/ckeditor5-engine/src/model/node';
import type { default as ModelElement } from '@ckeditor/ckeditor5-engine/src/model/element';
import type { ElementAttributes as ViewElementAttributes } from '@ckeditor/ckeditor5-engine/src/view/element';
import { type MatcherPattern as ViewMatcherPattern } from '@ckeditor/ckeditor5-engine/src/view/matcher';
import { type PriorityString } from '@ckeditor/ckeditor5-utils';

/**
 * The list of allowed attributes for CountDown.
 */
export const attributes: Record<string, string> = {
  class: 'class',
  date: 'date',
  alarm: 'alarm',
};

/**
 * The list of allowed attributes for CountDown.
 */
export const defaultOption: Record<string, string|number|boolean|null> = {
  class: '',
  date: null,
  alarm: null,
};

function convertModelAttributeToString(modelElement: ModelElement, attribute: string): string {
  let rawValue = modelElement.getAttribute(attribute);
  let value: string|undefined = undefined;

  if (rawValue !== defaultOption[attribute] && rawValue !== null) {
    if (typeof rawValue === 'number') {
      value = rawValue.toString();
    }
    else if (typeof rawValue === 'string') {
      if (rawValue !== '') {
        value = rawValue;
      }
    }
  }

  return value;
}

const createCountDownView: DowncastElementCreatorFunction = (modelElement, { writer }) => {
  // Re-use class from CountUp.
  const rootClasses = ['widget', 'countup-widget'];
  const modelClasses = convertModelAttributeToString(modelElement, 'class');

  if (modelClasses) {
    modelClasses.split(/\s+/).forEach((modelClass) => {
      if (!rootClasses.includes(modelClass)) {
        rootClasses.push(modelClass);
      }
    });
  }

  const date = convertModelAttributeToString(modelElement, 'date').replaceAll('|', ':');

  return writer.createContainerElement(
    'span',
    { class: rootClasses.join(' ') } as ViewElementAttributes,
    [
      writer.createContainerElement(
        'span',
        // Re-use class from CountUp.
        {class: 'countup-widget-endpoint'} as ViewElementAttributes,
        [writer.createText(date)]
      )
    ]
  );
}

export default class CountDownEditing extends Plugin {

  init() {
    this._defineSchema();
    this._defineConverters();
  }

  _defineSchema() {
    const schema = this.editor.model.schema;
    schema.register('countdown-timer', {
      inheritAllFrom: '$inlineObject',
      isObject: true,
      allowAttributes: Object.keys(attributes)
    });
    this.editor.editing.view.domConverter.inlineObjectElements.push('countdown-timer');
  }

  _defineConverters() {
    const conversion = this.editor.conversion;

    conversion
      .for('dataDowncast')
      .elementToElement({
        model: 'countdown-timer',
        view: (modelElement, conversionApi) => {
          const { writer } = conversionApi;
          const viewAttributes: ViewElementAttributes = {};

          Object.entries(attributes).forEach(([modelKey, viewKey]) => {
            let value = convertModelAttributeToString(modelElement, modelKey);

            if (value !== undefined) {
              if (modelKey === 'date') {
                // We need to replace the colon in date with some neutral
                // character, otherwise the date is removed by the Drupal XSS
                // filter. The colons are restored by the CountUp filter after
                // the HTML filter has finished processing.
                value = value.replaceAll(':', '|');
              }
              viewAttributes[viewKey] = value;
            }
          });

          return writer.createContainerElement('countdown-timer', viewAttributes, {
            renderUnsafeAttributes: Object.values(attributes)
          });
        }
      });

    conversion
      .for('editingDowncast')
      .elementToElement({
        model: 'countdown-timer',
        view: (modelElement, conversionApi, data) => {
          const countDownView = createCountDownView(modelElement, conversionApi, data);
          return toWidget(countDownView, conversionApi.writer, { label: 'CountDown' });
        }
      });

    const elementUpcastModel: UpcastElementCreatorFunction = (viewElement, { writer }) => {
      const modelAttributes: NodeAttributes = {};

      Object.entries(attributes).forEach(([modelAttribute, viewAttribute]) => {
        let value: string|number|boolean|null|undefined = undefined;

        if (viewElement.hasAttribute(viewAttribute)) {
          const rawValue = viewElement.getAttribute(viewAttribute);
          value = rawValue;

          if (typeof defaultOption[modelAttribute] === 'boolean') {
            value = !!rawValue;
          }
          // CountDown only supports integer.
          else if (typeof defaultOption[modelAttribute] === 'number') {
            value = parseInt(rawValue);
          }
        }

        if (value === undefined || (typeof value === 'number' && isNaN(value))) {
          value = defaultOption[modelAttribute];
        }

        modelAttributes[modelAttribute] = value;
      });

      return writer.createElement('countdown-timer', modelAttributes);
    };

    // noinspection TypeScriptValidateJSTypes
    conversion
      .for('upcast')
      .elementToElement({
        view: ({
          name: 'countdown-timer',
          attributes: {
            date: true,
          }
        }) as ViewMatcherPattern,
        model: elementUpcastModel,
        converterPriority: ('low') as PriorityString
      });
  }

}
