import { Plugin } from 'ckeditor5/src/core';
import { toWidget } from 'ckeditor5/src/widget';
import { type ElementCreatorFunction as DowncastElementCreatorFunction } from '@ckeditor/ckeditor5-engine/src/conversion/downcasthelpers';
import { type ElementCreatorFunction as UpcastElementCreatorFunction } from '@ckeditor/ckeditor5-engine/src/conversion/upcasthelpers';
import { type NodeAttributes } from '@ckeditor/ckeditor5-engine/src/model/node';
import type { default as ModelElement } from '@ckeditor/ckeditor5-engine/src/model/element';
import type { ElementAttributes as ViewElementAttributes } from '@ckeditor/ckeditor5-engine/src/view/element';
import { type MatcherPattern as ViewMatcherPattern } from '@ckeditor/ckeditor5-engine/src/view/matcher';
import { type PriorityString } from '@ckeditor/ckeditor5-utils';

/**
 * The list of allowed attributes for CountUp.
 */
export const attributes: Record<string, string> = {
  class: 'class',
  start: 'start',
  end: 'end',
  decimalPlaces: 'decimal-places',
  duration: 'duration',
  useEasing: 'use-easing',
  smartEasingThreshold: 'smart-easing-threshold',
  smartEasingAmount: 'smart-easing-amount',
  useGrouping: 'use-grouping',
  separator: 'separator',
  decimal: 'decimal',
  prefix: 'prefix',
  suffix: 'suffix'
};

/**
 * The list of allowed attributes for CountUp.
 */
export const defaultOption: Record<string, {
  type: string,
  value: string|number|boolean|null
}> = {
  class: {
    type: 'string',
    value: ''
  },
  start: {
    type: 'float',
    value: 0
  },
  end: {
    type: 'float',
    value: null
  },
  decimalPlaces: {
    type: 'integer',
    value: 0
  },
  duration: {
    type: 'float',
    value: 1
  },
  useEasing: {
    type: 'boolean',
    value: true
  },
  smartEasingThreshold: {
    type: 'integer',
    value: null
  },
  smartEasingAmount: {
    type: 'integer',
    value: null
  },
  useGrouping: {
    type: 'boolean',
    value: true
  },
  separator: {
    type: 'string',
    value: ','
  },
  decimal: {
    type: 'string',
    value: '.'
  },
  prefix: {
    type: 'string',
    value: ''
  },
  suffix: {
    type: 'string',
    value: ''
  }
};

function convertModelAttributeToString(modelElement: ModelElement, attribute: string): string {
  let rawValue = modelElement.getAttribute(attribute);
  let value: string|undefined = undefined;

  if ((['start', 'end'].includes(attribute) || rawValue !== defaultOption[attribute].value) && rawValue !== null) {
    if (typeof rawValue === 'number') {
      value = rawValue.toString();
    }
    else if (typeof rawValue === 'boolean') {
      if (rawValue) {
        value = 'true';
      }
    }
    else if (typeof rawValue === 'string') {
      if (rawValue !== '') {
        value = rawValue;
      }
    }
  }

  return value;
}

const createCountUpView: DowncastElementCreatorFunction = (modelElement, { writer }) => {
  const rootClasses = ['widget', 'countup-widget'];
  const modelClasses = convertModelAttributeToString(modelElement, 'class');

  if (modelClasses) {
    modelClasses.split(/\s+/).forEach((modelClass) => {
      if (!rootClasses.includes(modelClass)) {
        rootClasses.push(modelClass);
      }
    });
  }

  return writer.createContainerElement(
    'span',
    { class: rootClasses.join(' ') } as ViewElementAttributes,
    [
      writer.createContainerElement(
        'span',
        {class: 'countup-widget-endpoint'} as ViewElementAttributes,
        [writer.createText(convertModelAttributeToString(modelElement, 'start'))]
      ),
      writer.createContainerElement(
        'span',
        {} as ViewElementAttributes,
        [writer.createText('→')]
      ),
      writer.createContainerElement(
        'span',
        {class: 'countup-widget-endpoint'} as ViewElementAttributes,
        [writer.createText(convertModelAttributeToString(modelElement, 'end'))]
      )
    ]
  );
}

export default class CountUpEditing extends Plugin {

  init() {
    this._defineSchema();
    this._defineConverters();
  }

  _defineSchema() {
    const schema = this.editor.model.schema;
    schema.register('countup', {
      inheritAllFrom: '$inlineObject',
      isObject: true,
      allowAttributes: Object.keys(attributes)
    });
    this.editor.editing.view.domConverter.inlineObjectElements.push('countup');
  }

  _defineConverters() {
    const conversion = this.editor.conversion;

    conversion
      .for('dataDowncast')
      .elementToElement({
        model: 'countup',
        view: (modelElement, conversionApi) => {
          const { writer } = conversionApi;
          const viewAttributes: ViewElementAttributes = {};

          Object.entries(attributes).forEach(([modelKey, viewKey]) => {
            let value = convertModelAttributeToString(modelElement, modelKey);

            if (value !== undefined) {
              viewAttributes[viewKey] = value;
            }
          });

          return writer.createContainerElement('countup', viewAttributes, {
            renderUnsafeAttributes: Object.values(attributes)
          });
        }
      });

    conversion
      .for('editingDowncast')
      .elementToElement({
        model: 'countup',
        view: (modelElement, conversionApi, data) => {
          const countUpView = createCountUpView(modelElement, conversionApi, data);
          return toWidget(countUpView, conversionApi.writer, { label: 'CountUp' });
        }
      });

    const elementUpcastModel: UpcastElementCreatorFunction = (viewElement, { writer }) => {
      const modelAttributes: NodeAttributes = {};

      Object.entries(attributes).forEach(([modelAttribute, viewAttribute]) => {
        let value: string|number|boolean|null|undefined = undefined;

        if (viewElement.hasAttribute(viewAttribute)) {
          const rawValue = viewElement.getAttribute(viewAttribute);

          switch (defaultOption[modelAttribute].type) {
            case 'boolean':
              value = !!rawValue;
              break

            case 'float':
              value = parseFloat(rawValue);
              break

            case 'integer':
              value = parseInt(rawValue);
              break

            default:
              value = rawValue;
          }
        }

        if (value === undefined || (typeof value === 'number' && isNaN(value))) {
          value = defaultOption[modelAttribute].value;
        }

        modelAttributes[modelAttribute] = value;
      });

      return writer.createElement('countup', modelAttributes);
    };

    // noinspection TypeScriptValidateJSTypes
    conversion
      .for('upcast')
      .elementToElement({
        view: ({
          name: 'countup',
          attributes: {
            start: true,
            end: true
          }
        }) as ViewMatcherPattern,
        model: elementUpcastModel,
        converterPriority: ('low') as PriorityString
      });
  }

}
