const { abs, floor } = Math;

export const UNITS = ['day', 'hour', 'minute', 'second'];
export const ISO_8601 = /^\d{4}(-\d\d(-\d\d(T\d\d:\d\d(:\d\d)?(\.\d+)?(([+-]\d\d:\d\d)|Z)?)?)?)?$/i;
const MS_IN_A_SECOND = 1000;
const MS_IN_A_MINUTE = 60 * MS_IN_A_SECOND;
const MS_IN_A_HOUR = 60 * MS_IN_A_MINUTE;
const MS_IN_A_DAY = 24 * MS_IN_A_HOUR;

const LABELS_DEFAULT = ['Days', 'Hours', 'Minutes', 'Seconds'];

export default class CountDownTimer extends HTMLElement {

  static get observedAttributes() {
    return ['alarm', 'date', 'labels'];
  }

  constructor() {
    super();
    this.state = {
      date: new Date().toISOString(),
      alarm: '',
      labels: '',
      error: '',
    };
  }

  setState(newState) {
    this.state = {
      ...this.state,
      ...newState,
    };
    this.updateView();
  }

  connectedCallback() {
    const shadow = this.attachShadow({ mode: 'open' });
    this.element = document.createElement('div');
    this.checkAttributes({ date: this.getAttribute('date') });
    this.setState({
      alarm: this.getAttribute('alarm'),
      date: this.getAttribute('date'),
      labels: this.getAttribute('labels'),
    });
    this.update(new Date(this.state.date) - new Date());
    this.tick = setInterval(() => {
      const counter = new Date(this.state.date) - new Date();
      this.update(counter);
    }, 1_000);
    const style = document.createElement('style');
    style.textContent = `
      :host {
        display: inline-block;
      }
      div {
        display: inline-grid;
        justify-content: center;
        grid-auto-flow: column;
        gap: clamp(.5rem, .15em, 1rem);
        font-size: var(--cdt-digit-font-size, 3rem);
        line-height: 1;
      }
      span {
        font-size: 1rem;
      }
      .pulse {
        animation: ease-in-out pulse .25s alternate infinite;
      }
      @keyframes pulse {
        from {
          scale: .98;
        }
        to {
          scale: 1;
        }
      }
    `;
    shadow.appendChild(style);
    shadow.appendChild(this.element);
  }

  updateView() {
    if (this.element) {
      if (this.state.error) {
        this.element.innerHTML = `<span>${this.state.error}</span>`;
      } else {
        const labels = LABELS_DEFAULT;

        if (typeof this.state.labels === 'string' && this.state.labels !== '') {
          this.state.labels.split('|', 4).forEach((label, index) => {
            const trimmed = label.trim();

            if (trimmed.length) {
              labels[index] = trimmed;
            }
          })
        }

        this.element.innerHTML = UNITS.map(
          (unit, index) => `<ct-number exportparts="number-label, digit" label="${labels[index]}" unit="${unit}" number="0"></ct-number>`,
        ).join('');
      }
    }
  }

  checkAttributes({ date }) {
    if (date === undefined) return;
    if (date === '') {
      this.setState({ error: 'No date provided.' });
      return;
    }
    if (!date.match(ISO_8601)) {
      this.setState({ error: 'No valid ISO 8601 date format provided.' });
      return;
    }
    this.setState({ error: '' });
  }

  format(ms, unit) {
    let value;
    switch (unit) {
      case 'day':
        value = ms / MS_IN_A_DAY;
        break;
      case 'hour':
        value = (ms % MS_IN_A_DAY) / MS_IN_A_HOUR;
        break;
      case 'minute':
        value = (ms % MS_IN_A_HOUR) / MS_IN_A_MINUTE;
        break;
      case 'second':
        value = (ms % MS_IN_A_MINUTE) / MS_IN_A_SECOND;
        break;
      default:
        value = 0;
        break;
    }
    if (unit === 'day') return floor(value).toString();
    return floor(value).toString().padStart(2, '0');
  }

  update(counter) {
    if (counter <= 0) {
      clearInterval(this.tick);
      counter = 0;
    }
    UNITS.map((unit) => {
      const element = this.element.querySelector(`[unit="${unit}"]`);
      element?.setAttribute('number', this.format(abs(counter), unit));
    });
    const timeLeft = counter / MS_IN_A_SECOND;
    this.element.classList.toggle(
      'pulse',
      timeLeft < this.state.alarm && timeLeft > 0,
    );
  }

  attributeChangedCallback(name, oldValue, newValue) {
    this.checkAttributes({ [name]: newValue });
    this.setState({ [name]: newValue });
  }
}
