(function (window, Drupal, once, debounce, countUp)  {

  'use strict';

  /**
   * Attach the countUp.js behaviour to the expected elements.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.countup = {

    /**
     * The selector to be used for CountUp targets.
     *
     * @var string
     */
    countUpSelector: '.countup',

    /**
     * The list of events applied to the window element.
     *
     * @var array
     */
    events: ['DOMContentLoaded', 'load', 'resize', 'scroll'],

    /**
     * The event callback.
     */
    eventCallback: null,

    /**
     * @inheritdoc
     */
    attach(context) {
      if (typeof countUp.CountUp !== 'function') {
        return;
      }

      if (once('countup', '.countup', context).length) {
        if (!this.eventCallback) {
          this.eventCallback = debounce(this.checkVisibility.bind(this), 500);

          this.events.forEach((event) => {
            window.addEventListener(event, this.eventCallback, false);
          });
        }

        // Force immediate execution for already visible elements.
        this.checkVisibility();
      }
    },

    /**
     * @inheritdoc
     */
    detach() {
      this.events.forEach((event) => {
        window.removeEventListener(event, this.eventCallback, false);
      });
    },

    /**
     * Check whether an element is visible or not.
     *
     * @param {HTMLElement} element
     *
     * @return {boolean}
     */
    isElementVisible(element) {
      return !!((element.offsetWidth > 0 && element.offsetHeight > 0) || element.getClientRects().length);
    },

    /**
     * Check whether an element is in the viewport.
     *
     * @param {HTMLElement} element
     *
     * @return {boolean}
     */
    isElementInViewport(element) {
      const rect = element.getBoundingClientRect();

      return rect.top >= 0 &&
        rect.left >= 0 &&
        rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
        rect.right <= (window.innerWidth || document.documentElement.clientWidth);
    },

    /**
     * Returns the opacity of the element, which may be inherited from its parents.
     *
     * @param {HTMLElement|Node} element
     *
     * @return {number}
     */
    getElementOpacity(element) {
      for (; element && element !== document; element = element.parentNode) {
        const opacity = parseFloat(getComputedStyle(element).opacity);

        if (getComputedStyle(element).opacity < 1) {
          return opacity;
        }
      }

      return 1;
    },

    checkVisibility: function() {
      const elements = document.querySelectorAll(this.countUpSelector + ':not(.countup-processed)');

      [].forEach.call(elements, (element) => {
        if (this.isElementVisible(element) && this.getElementOpacity(element) > 0 && this.isElementInViewport(element)) {
          element.classList.add('countup-processed');
          this.countUp(element);
        }
      });
    },

    /**
     * Performs the countUp action.
     *
     * @param {HTMLElement} element
     */
    countUp(element) {
      const options = Object.assign({}, { start: 0, duration: 2.5, decimalPlaces: 0}, element.dataset);

      if (typeof options.end === 'undefined') {
        // Check if the inner HTML is a valid number.
        if (!isNaN(element.innerHTML - parseFloat(element.innerHTML))) {
          options.end = element.innerHTML;
          element.setAttribute('data-end', options.end);
        }
        else {
          // Nothing valid is defined for the "end" value, exit here...
          return;
        }
      }

      // Instantiate the plugin.
      const count = new countUp.CountUp(element, options.end, options);

      if (!count.error) {
        count.start();
      }
      else if (window.console) {
        window.console.error(count.error);
      }
    }

  };

})(window, window.Drupal, window.once, window.Drupal.debounce, window.countUp);
