<?php

declare(strict_types=1);

namespace Drupal\countup\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Ajax\EditorDialogSave;

/**
 * Provides a CountUp form.
 */
final class CountDownEditorForm extends FormBase {

  public function defaultConfiguration(): array {
    return [
      'date' => NULL,
      'alarm' => 60,
      'class' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'countup_editor_countdown';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $configuration = $this->getRequest()->request->all('editor_object') + $this->defaultConfiguration();
    // @todo Remove the ID when we can use selectors to replace content via
    //   AJAX in https://www.drupal.org/project/drupal/issues/2821793.
    $form['#prefix'] = '<div id="countup-editor-countdown-form-wrapper">';
    $form['#suffix'] = '</div>';

    if (!empty($configuration['date'])) {
      //$date = strtr($configuration['date'], '|', ':');
      $configuration['date'] = new DrupalDateTime($configuration['date']);
    }

    $form['date'] = [
      '#title' => $this->t('Date'),
      '#description' => $this->t('The date when the timer stops.'),
      '#type' => 'datetime',
      '#default_value' => $configuration['date'],
      '#required' => TRUE,
    ];

    $form['alarm'] = [
      '#title' => $this->t('Alarm'),
      '#description' => $this->t('The number of seconds before the end of the timer, from which the latter starts to pulse.'),
      '#type' => 'number',
      '#default_value' => $configuration['alarm'],
      '#step' => 1,
      '#size' => 8,
      '#required' => TRUE,
    ];

    $form['class'] = [
      '#title' => $this->t('Extra classes'),
      '#description' => $this->t('A list of classes to be applied to the element, separated by a space.'),
      '#type' => 'textfield',
      '#default_value' => $configuration['class'],
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
        '#ajax' => [
          'callback' => '::updateWidget',
          'wrapper' => 'countup-editor-countdown-form-wrapper',
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * AJAX callback to send the configuration to the CKEditor UI.
   *
   * @param array $form
   *   The complete form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return array|\Drupal\Core\Ajax\AjaxResponse
   *   The form array when there are form errors or an AJAX response when the
   *   configuration is finished.
   */
  public function updateWidget(array &$form, FormStateInterface $form_state): AjaxResponse|array {
    if ($form_state::hasAnyErrors()) {
      return $form;
    }

    $alarm = $form_state->getValue('alarm');
    $date = $form_state->getValue('date')->format('c');
    // Remove any colon, otherwise the Drupal XSS filter will break the date.
    //$date = strtr($date, ':', '|');

    $values = [
      'date' => $date,
      'alarm' => $alarm !== '' ? (int) $alarm : 0,
      'class' => $form_state->getValue('class'),
    ];

    $response = new AjaxResponse();
    $response->addCommand(new EditorDialogSave($values));
    $response->addCommand(new CloseDialogCommand());
    return $response;
  }

}
