<?php

declare(strict_types=1);

namespace Drupal\countup\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Ajax\EditorDialogSave;

/**
 * Provides a CountUp form.
 */
final class CountUpEditorForm extends FormBase {

  public function defaultConfiguration(): array {
    return [
      'start' => 0,
      'end' => NULL,
      'decimal_places' => 0,
      'duration' => 1,
      'use_easing' => TRUE,
      'smart_easing_threshold' => NULL,
      'smart_easing_amount' => NULL,
      'use_grouping' => TRUE,
      'separator' => ',',
      'decimal' => '.',
      'class' => '',
      'prefix' => '',
      'suffix' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'countup_editor_countup';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $configuration = $this->getRequest()->request->all('editor_object') + $this->defaultConfiguration();
    // @todo Remove the ID when we can use selectors to replace content via
    //   AJAX in https://www.drupal.org/project/drupal/issues/2821793.
    $form['#prefix'] = '<div id="countup-editor-countup-form-wrapper">';
    $form['#suffix'] = '</div>';

    $form['start'] = [
      '#title' => $this->t('Start number'),
      '#type' => 'number',
      '#default_value' => $configuration['start'],
      '#step' => 'any',
      '#size' => 8,
      '#required' => TRUE,
    ];

    $form['end'] = [
      '#title' => $this->t('End number'),
      '#type' => 'number',
      '#default_value' => $configuration['end'],
      '#step' => 'any',
      '#size' => 8,
      '#required' => TRUE,
    ];

    $form['decimal_places'] = [
      '#title' => $this->t('Number of decimals places'),
      '#type' => 'number',
      '#default_value' => $configuration['decimal_places'],
      '#min' => 0,
      '#step' => 1,
      '#required' => TRUE,
    ];

    $form['duration'] = [
      '#title' => $this->t('Duration of animation'),
      '#type' => 'number',
      '#default_value' => $configuration['duration'],
      '#field_suffix' => 'seconds',
      '#min' => 0,
      '#step' => .1,
      '#required' => TRUE,
    ];

    $form['use_easing'] = [
      '#title' => $this->t('Use easing'),
      '#type' => 'checkbox',
      '#default_value' => !empty($configuration['use_easing']),
    ];

    $form['smart_easing_threshold'] = [
      '#title' => $this->t('Smart easing threshold'),
      '#type' => 'textfield',
      '#default_value' => $configuration['smart_easing_threshold'],
      '#size' => 4,
      '#description' => $this->t('Smooth easing for large numbers above this if useEasing (Default to 999).'),
      '#states' => [
        'disabled' => [
          ':input[name="use_easing"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['smart_easing_amount'] = [
      '#title' => $this->t('smart easing amount'),
      '#type' => 'textfield',
      '#default_value' => $configuration['smart_easing_amount'],
      '#size' => 4,
      '#description' => $this->t('Amount to be eased for numbers above threshold (Default to 333).'),
      '#states' => [
        'disabled' => [
          ':input[name="attributes[use_easing]"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['use_grouping'] = [
      '#title' => $this->t('Group the element'),
      '#type' => 'checkbox',
      '#default_value' => !empty($configuration['use_grouping']),
      '#description' => $this->t('example: 1,000 vs 1000 (true)'),
    ];

    $form['separator'] = [
      '#title' => $this->t('Thousand separator'),
      '#type' => 'textfield',
      '#default_value' => $configuration['separator'],
      '#size' => 2,
      '#states' => [
        'disabled' => [
          ':input[name="attributes[use_grouping]"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['decimal'] = [
      '#title' => $this->t('Decimal symbol'),
      '#type' => 'textfield',
      '#default_value' => $configuration['decimal'],
      '#size' => 2,
      '#states' => [
        'disabled' => [
          ':input[name="attributes[decimal_places]"]' => ['value' => '0'],
        ],
      ],
    ];

    $form['class'] = [
      '#title' => $this->t('Extra classes'),
      '#description' => $this->t('A list of classes to be applied to the element, separated by a space.'),
      '#type' => 'textfield',
      '#default_value' => $configuration['class'],
    ];

    $form['prefix'] = [
      '#title' => $this->t('Prefix'),
      '#type' => 'textfield',
      '#default_value' => $configuration['prefix'],
      '#size' => 10,
    ];

    $form['suffix'] = [
      '#title' => $this->t('Suffix'),
      '#type' => 'textfield',
      '#default_value' => $configuration['suffix'],
      '#size' => 10,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
        '#ajax' => [
          'callback' => '::updateWidget',
          'wrapper' => 'countup-editor-countup-form-wrapper',
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * AJAX callback to send the configuration to the CKEditor UI.
   *
   * @param array $form
   *   The complete form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return array|\Drupal\Core\Ajax\AjaxResponse
   *   The form array when there are form errors or an AJAX response when the
   *   configuration is finished.
   */
  public function updateWidget(array &$form, FormStateInterface $form_state): AjaxResponse|array {
    if ($form_state::hasAnyErrors()) {
      return $form;
    }

    $smartEasingThreshold = $form_state->getValue('smart_easing_threshold');
    $smartEasingAmount = $form_state->getValue('smart_easing_amount');

    $values = [
      'start' => (float) $form_state->getValue('start'),
      'end' => (float) $form_state->getValue('end'),
      'decimalPlaces' => (int) $form_state->getValue('decimal_places'),
      'duration' => (float) $form_state->getValue('duration'),
      'useEasing' => (bool) $form_state->getValue('use_easing'),
      'smartEasingThreshold' => $smartEasingThreshold !== '' ? (int) $smartEasingThreshold : NULL,
      'smartEasingAmount' => $smartEasingAmount !== '' ? (int) $smartEasingAmount : NULL,
      'useGrouping' => (bool) $form_state->getValue('use_grouping'),
      'separator' => $form_state->getValue('separator'),
      'decimal' => $form_state->getValue('decimal'),
      'class' => $form_state->getValue('class'),
      'prefix' => $form_state->getValue('prefix'),
      'suffix' => $form_state->getValue('suffix'),
    ];

    $response = new AjaxResponse();
    $response->addCommand(new EditorDialogSave($values));
    $response->addCommand(new CloseDialogCommand());
    return $response;
  }

}
