<?php

namespace Drupal\countup\Plugin\Filter;

use Drupal\Component\Utility\Html;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * Provides a filter to attach the needed libraries for the CountUp plugin.
 *
 * @Filter(
 *   id = "countup",
 *   title = @Translation("CountUp"),
 *   description = @Translation("Process <countup> tag and attach the required libraries for the CountUp plugin."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE
 * )
 */
class CountUp extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $result = new FilterProcessResult($text);

    if (stristr($text, 'countdown') !== FALSE) {
      $translations = implode('|', [
        t('Days', [], ['context' => 'CountDown timer']),
        t('Hours', [], ['context' => 'CountDown timer']),
        t('Minutes', [], ['context' => 'CountDown timer']),
        t('Seconds', [], ['context' => 'CountDown timer']),
      ]);

      $labelRemove = fn($string) => preg_replace('/\s*labels=(["\'])[^\1]*\1\s*/', ' ', $string);

      // Restore the date value, as the HTML filter should have run before, if
      // enabled.
      $text = preg_replace_callback('/(<countdown-timer[^>]+date=)(["\'])([^\2]+)(\2)([^>]*>)/', function(array $matches) use ($translations, $labelRemove) {
        $matches[1] = $labelRemove($matches[1]);
        $matches[5] = $labelRemove($matches[5]);
        return $matches[1] . $matches[2] . strtr($matches[3], '|', ':') . $matches[4] . ' labels="' . $translations . '"' . $matches[5];
      }, $text);

      $result->setProcessedText($text);
      $result->addAttachments([
        'library' => ['countup/countdown'],
      ]);
    }

    // @todo rewrite the CountUp JS as a custom HTML element, which will allow
    //   to remove most of the code below.
    if (stristr($text, 'countup') !== FALSE) {
      $dom = Html::load($text);
      $hasValidCountUp = FALSE;

      /** @var \DOMNode $node */
      foreach ($dom->getElementsByTagName('countup') as $node) {
        $nodeIsValidCountUp = FALSE;

        if ($node instanceof \DOMElement && $node->hasAttributes()) {
          $classes = [];
          $span = $dom->createElement('span', '&nbsp;');
          // Force "start" attribute to the default value. It should normally be
          // overridden just after.
          $span->setAttribute('data-start', '0');

          /** @var \DomNode $attribute */
          foreach ($node->attributes as $attribute) {
            if ($attribute->nodeName === 'class') {
              $classes = preg_split('/\s+/', $attribute->nodeValue);
            }
            else {
              $span->setAttribute('data-' . $attribute->nodeName, $attribute->nodeValue);

              // The "end" attribute is required and has no default value.
              if ($attribute->nodeName === 'end') {
                $span->nodeValue = $attribute->nodeValue;
                $nodeIsValidCountUp = TRUE;
              }
            }
          }

          $classes[] = 'countup';
          $span->setAttribute('class', implode(' ', array_unique($classes)));
          $node->replaceWith($span);
        }

        if ($nodeIsValidCountUp) {
          $hasValidCountUp = TRUE;
        }
        else {
          $node->parentNode->removeChild($node);
        }
      }

      $result->setProcessedText(Html::serialize($dom));

      if ($hasValidCountUp) {
        $result->addAttachments([
          'library' => ['countup/core'],
        ]);
      }
    }

    return $result;
  }

}
